//**************************************************//
// An example of an OpenGL animation for LCCWin-32  //
//  Author:  Josef Planeta    (planeta@iach.cz)    //
//**************************************************//



#include <windows.h>			/* must include this before GL/gl.h */
#include <GL/gl.h>				/* OpenGL header file */
#include <stdio.h>
#include <stdlib.h>


//prototypes:
void setupPixelFormat(HDC hDC);
void setupPalette(HDC hDC);


//global variables:
HDC 		hDC;				/* device context */
HPALETTE 	hPalette ;			/* custom palette (if needed) */
HGLRC 		hGLRC;				/* opengl context */
HWND        hWnd ;
MSG			msg ;

static		first_draw=1;		//flag for first drawing
float 		x=0.5,y=0,z=0;		//vectors of rotation
signed int	sx=1,sy=1,sz=1;		//signs of rotation


// *********************************************
// *            Procedures for OpenGL          *
// *********************************************

void setupPixelFormat(HDC hDC)
{
    PIXELFORMATDESCRIPTOR pfd = {
        sizeof(PIXELFORMATDESCRIPTOR),  // size
        1,                              // version
        PFD_SUPPORT_OPENGL |
        PFD_DRAW_TO_WINDOW |
        PFD_DOUBLEBUFFER,               // support double-buffering
        PFD_TYPE_RGBA,                  // color type
        16,                             // prefered color depth
        0, 0, 0, 0, 0, 0,               // color bits (ignored)
        0,                              // no alpha buffer
        0,                              // alpha bits (ignored)
        0,                              // no accumulation buffer
        0, 0, 0, 0,                     // accum bits (ignored)
        16,                             // depth buffer
        0,                              // no stencil buffer
        0,                              // no auxiliary buffers
        PFD_MAIN_PLANE,                 // main layer
        0,                              // reserved
        0, 0, 0,                        // no layer, visible, damage masks
    };
    int pixelFormat;

    pixelFormat = ChoosePixelFormat(hDC, &pfd);
    if (pixelFormat == 0) {
        MessageBox(WindowFromDC(hDC), "ChoosePixelFormat failed.", "Error",
                MB_ICONERROR | MB_OK);
        exit(1);
    }

    if (SetPixelFormat(hDC, pixelFormat, &pfd) != TRUE) {
        MessageBox(WindowFromDC(hDC), "SetPixelFormat failed.", "Error",
                MB_ICONERROR | MB_OK);
        exit(1);
    }
}

//--------------------------------------------------------------/

void setupPalette(HDC hDC)
{
    int pixelFormat = GetPixelFormat(hDC);
    PIXELFORMATDESCRIPTOR pfd;
    LOGPALETTE* pPal;
    int paletteSize;

    DescribePixelFormat(hDC, pixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);

    if (pfd.dwFlags & PFD_NEED_PALETTE) {
        paletteSize = 1 << pfd.cColorBits;
    } else {
        return;
    }

    pPal = (LOGPALETTE*)
        malloc(sizeof(LOGPALETTE) + paletteSize * sizeof(PALETTEENTRY));
    pPal->palVersion = 0x300;
    pPal->palNumEntries = paletteSize;

    // build a simple RGB color palette
    {
        int redMask = (1 << pfd.cRedBits) - 1;
        int greenMask = (1 << pfd.cGreenBits) - 1;
        int blueMask = (1 << pfd.cBlueBits) - 1;
        int i;

        for (i=0; i<paletteSize; ++i) {
            pPal->palPalEntry[i].peRed =
                    (((i >> pfd.cRedShift) & redMask) * 255) / redMask;
            pPal->palPalEntry[i].peGreen =
                    (((i >> pfd.cGreenShift) & greenMask) * 255) / greenMask;
            pPal->palPalEntry[i].peBlue =
                    (((i >> pfd.cBlueShift) & blueMask) * 255) / blueMask;
            pPal->palPalEntry[i].peFlags = 0;
        }
    }

    hPalette = CreatePalette(pPal);
    free(pPal);

    if (hPalette) {
        SelectPalette(hDC, hPalette, FALSE);
        RealizePalette(hDC);
    }
}

//--------------------------------------------------------------/
void
init(void)
{
    // set viewing projection
    glMatrixMode(GL_PROJECTION);
    glFrustum(-.5, .5, -.5, .5, 1, 6);

    // position viewer
    glMatrixMode(GL_MODELVIEW);
    glTranslatef(0.0F, 0.0F, -2.0F);

    // position object
    glRotatef(30.0F, 1.0F, 0.0F, 0.0F);
    glRotatef(30.0F, 0.0F, 1.0F, 0.0F);


    glEnable(GL_DEPTH_TEST);
//    glEnable(GL_LIGHTING);
//    glEnable(GL_LIGHT0);
}


//**************** end of the Open GL procedures ***************************


// --------------- Procedures ----------------------------


void display(void)
{

int f=0;					//flag

if (first_draw)				//first draw only once!
    {
    first_draw=0;
    f=1;
    }

do{
    // clear color and depth buffers
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


// Now, let's do some chaos in rotation

    if (rand() > 30000)
    {x=x + sx*0.05;
    sx=(x>1) ? -sx : sx;
    sx=(x<-1) ? -sx : sx;
    }

    if (rand() > 30000)
    {y=y + sy*0.05;
    sy=(y>1) ? -sy : sy;
    sy=(y<-1) ? -sy : sy;
    }

    if (rand() > 30000)
    {z=z + sz*0.05;
    sz=(z>1) ? -sz : sz;
    sz=(z<-1) ? -sz : sz;
    }



    glRotatef(1,x,y,z);


    // draw six faces of a cube
    glBegin(GL_QUADS);
    glNormal3f( 0.0F, 0.0F, 1.0F);
    glColor3f(0,0,1);
    glVertex3f( 0.5F, 0.5F, 0.5F);
    glColor3f(0,1,0);
    glVertex3f(-0.5F, 0.5F, 0.5F);
    glColor3f(1,1,0);
    glVertex3f(-0.5F,-0.5F, 0.5F);
    glColor3f(1,0,0);
    glVertex3f( 0.5F,-0.5F, 0.5F);

    glNormal3f( 0.0F, 0.0F,-1.0F);
    glColor3f(1,0,1);
    glVertex3f(-0.5F,-0.5F,-0.5F);
    glColor3f(0,0,1);
    glVertex3f(-0.5F, 0.5F,-0.5F);
    glColor3f(1,1,0);
    glVertex3f( 0.5F, 0.5F,-0.5F);
    glColor3f(1,0,0);
    glVertex3f( 0.5F,-0.5F,-0.5F);

    glNormal3f( 0.0F, 1.0F, 0.0F);
    glColor3f(0,0,1);
    glVertex3f( 0.5F, 0.5F, 0.5F);
    glColor3f(1,0,0);
    glVertex3f( 0.5F, 0.5F,-0.5F);
    glColor3f(1,0,1);
    glVertex3f(-0.5F, 0.5F,-0.5F);
    glColor3f(0,1,0);
    glVertex3f(-0.5F, 0.5F, 0.5F);

    glNormal3f( 0.0F,-1.0F, 0.0F);
    glColor3f(1,0,0);
    glVertex3f(-0.5F,-0.5F,-0.5F);
    glColor3f(0,1,0);
    glVertex3f( 0.5F,-0.5F,-0.5F);
    glColor3f(0,0,1);
    glVertex3f( 0.5F,-0.5F, 0.5F);
    glColor3f(1,1,0);
    glVertex3f(-0.5F,-0.5F, 0.5F);

    glNormal3f( 1.0F, 0.0F, 0.0F);
    glColor3f(1,0,1);
    glVertex3f( 0.5F, 0.5F, 0.5F);
    glColor3f(1,1,0);
    glVertex3f( 0.5F,-0.5F, 0.5F);
    glColor3f(0,1,1);
    glVertex3f( 0.5F,-0.5F,-0.5F);
    glColor3f(0,0,1);
    glVertex3f( 0.5F, 0.5F,-0.5F);

    glNormal3f(-1.0F, 0.0F, 0.0F);
    glColor3f(0,1,0);
    glVertex3f(-0.5F,-0.5F,-0.5F);
    glColor3f(1,1,1);
    glVertex3f(-0.5F,-0.5F, 0.5F);
    glColor3f(0,1,1);
    glVertex3f(-0.5F, 0.5F, 0.5F);
    glColor3f(1,0,0);
    glVertex3f(-0.5F, 0.5F,-0.5F);

    glEnd();


    glFlush();
    SwapBuffers(hDC);


    while (PeekMessage(&msg, hWnd, WM_LBUTTONDOWN, WM_LBUTTONDOWN, PM_REMOVE))
        {
        switch(msg.message)
            {
            case WM_LBUTTONDOWN:
                SetWindowText(hWnd,"Click 2 cube 4 START");
                f=1;
                break;
            }

        }					//while

    } while(!f);			//do

}							//display()





// -------------- WndProc()----------------------------//

LONG WINAPI
WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    static PAINTSTRUCT ps;

    switch(msg) {

    case WM_CREATE:
    // initialize OpenGL rendering
    hDC = GetDC(hWnd);
    setupPixelFormat(hDC);
    setupPalette(hDC);
    hGLRC = wglCreateContext(hDC);
    wglMakeCurrent(hDC, hGLRC);
    init();
    return 0;



    case WM_PAINT:
    display();
    BeginPaint(hWnd, &ps);
    EndPaint(hWnd, &ps);
    return 0;

    case WM_SIZE:
    glViewport(0, 0, LOWORD(lParam), HIWORD(lParam));
    PostMessage(hWnd, WM_PAINT, 0, 0);
    return 0;

    case WM_LBUTTONDOWN:
    SetWindowText(hWnd,"Click 2 cube 4 STOP");
    PostMessage(hWnd, WM_PAINT, 0, 0);
    return 0;

    case WM_PALETTECHANGED:
    if (hWnd == (HWND)wParam)
        break;
    // fall through to WM_QUERYNEWPALETTE

    case WM_QUERYNEWPALETTE:
    if (hPalette) {
        UnrealizeObject(hPalette);
        SelectPalette(hDC, hPalette, FALSE);
        RealizePalette(hDC);
        return TRUE;
    }
    return FALSE;

    case WM_CLOSE:
    PostQuitMessage(0);
    return 0;
    }

    return DefWindowProc(hWnd, msg, wParam, lParam);
}



// **************************************************
// *                                                *
// *                   WinMain()                    *
// *                                                *
// **************************************************


int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
LPSTR lpszCmdLine, int nCmdShow)
{

    static char szAppName[] = "OpenGL" ;
    WNDCLASS    wndclass ;

    DWORD buffer = PFD_DOUBLEBUFFER;	/* buffering type */
    BYTE  color  = PFD_TYPE_RGBA;		/* color type */


if (!hPrevInstance)
    {
    wndclass.style		= CS_HREDRAW | CS_VREDRAW ;
    wndclass.lpfnWndProc	= WndProc ;
    wndclass.cbClsExtra	= 0 ;
    wndclass.cbWndExtra	= 0 ;
    wndclass.hInstance	= hInstance ;
    wndclass.hIcon		= LoadIcon (NULL, IDI_APPLICATION) ;
    wndclass.hCursor	= LoadCursor (NULL, IDC_ARROW) ;
    wndclass.hbrBackground	= GetStockObject (WHITE_BRUSH) ;
    wndclass.lpszMenuName	= NULL ;
    wndclass.lpszClassName	= szAppName ;

    RegisterClass (&wndclass) ;
    }

    hWnd = CreateWindow (szAppName,		// window class name
        "Click 2 cube 4 STOP",			// window caption
        WS_OVERLAPPEDWINDOW,			// window style
        CW_USEDEFAULT,					// initial x position
        CW_USEDEFAULT,					// initial y position
        220, 							// initial x size
        220, 							// initial y size
        NULL,							// parent window handle
        NULL,							// window menu handle
        hInstance,						// program instance handle
        NULL) ;							// creation parameters


    if (hWnd == NULL)					//Window wasn't created!
        exit(1);						//ond of program (not fction)


    ShowWindow(hWnd, SW_SHOW);
    UpdateWindow(hWnd);


    //process messages
    while (GetMessage (&msg, NULL, 0, 0))
    {
    TranslateMessage (&msg) ;
    DispatchMessage (&msg) ;
    }

    //quit
    wglMakeCurrent(NULL, NULL);
    ReleaseDC(hDC, hWnd);
    wglDeleteContext(hGLRC);
    DestroyWindow(hWnd);
    if (hPalette)
    DeleteObject(hPalette);
    return 0;
}
